// DemoAddFolder.cpp : Defines the entry point for the console application.
//

#include <iostream>
using namespace std;

#include <COMUtils.h>
#include <shobjidl.h>
#include <shlobj.h>
#include <Knownfolders.h>

void ListFolders(SmartCOMPtr<IShellLibrary> library)
{
	HRESULT hr;
	SmartCOMPtr<IObjectArray> ioarr;

	hr = library->GetFolders(LFF_ALLITEMS,
		IID_IObjectArray,
		reinterpret_cast<void**>(&ioarr));

	if(hr == S_OK)
		cout << "Tous les repertoires sont obtenus" << endl;
	if(hr == S_FALSE)
		cout << "Des repertoires ne seront pas listes" << endl;
	if(hr != S_OK && hr != S_FALSE)
		COMHelper::TestOk(hr);

	UINT count;
	hr = ioarr->GetCount(&count);
	COMHelper::TestOk(hr);
	for(UINT i=0; i<count; i++)
	{
		SmartCOMPtr<IShellItem> item;
		SmartCOMPtr<IShellItem> resolved;
		WCHAR *nom;
		hr = ioarr->GetAt(i, IID_IShellItem,
			reinterpret_cast<void**>(&item));
		COMHelper::TestOk(hr);
		hr = item->GetDisplayName(SIGDN_NORMALDISPLAY , &nom);
		COMHelper::TestOk(hr);
		wcout << nom << " -> ";
		::CoTaskMemFree(nom);
	
		hr = library->ResolveFolder(&(*item),
			1000,
			IID_IShellItem,
			reinterpret_cast<void**>(&resolved));	
		if(hr != S_OK && hr != S_FALSE)
		{
			cout << endl;
			continue;
		}
		hr = resolved->GetDisplayName(SIGDN_FILESYSPATH, &nom);
		COMHelper::TestOk(hr);
		wcout << nom << endl;
		::CoTaskMemFree(nom);	
	}
}

SmartCOMPtr<IShellLibrary> GetLibrary(GUID guidlib)
{
	// Attention a STGM_READWRITE
	// car ajouter un rpertoire c'est
	// crire dans la librairie :-)
	HRESULT hr;
	SmartCOMPtr<IShellLibrary> library;

	hr = ::SHLoadLibraryFromKnownFolder(
		guidlib,
		STGM_READWRITE,
		IID_IShellLibrary,
		reinterpret_cast<void**>(&library));
	COMHelper::TestOk(hr);

	return library;
}

void ShowDefaultFolder(SmartCOMPtr<IShellLibrary> library)
{
	HRESULT hr;
	WCHAR *nom;

	SmartCOMPtr<IShellItem> item;

	cout << "Repertoire par defaut: ";
	hr = library->GetDefaultSaveFolder(
		DSFT_DETECT,
		IID_IShellItem,
		reinterpret_cast<void**>(&item));
	COMHelper::TestOk(hr);

	hr = item->GetDisplayName(SIGDN_NORMALDISPLAY , &nom);
	COMHelper::TestOk(hr);
	wcout << nom << " -> ";
	::CoTaskMemFree(nom);

	hr = item->GetDisplayName(SIGDN_FILESYSPATH, &nom);
	COMHelper::TestOk(hr);
	wcout << nom << endl;
	::CoTaskMemFree(nom);	
}

/*
* AddFolderAndSetDefault
*
* ATTENTION, si le rpertoire est dj prsent dans la librairie
* cette fonction choue
*/

SmartCOMPtr<IShellItem>
AddFolderAndSetDefault(SmartCOMPtr<IShellLibrary> library, WCHAR* folder)
{
	HRESULT hr;
	SmartCOMPtr<IShellItem> item;
	
	hr = ::SHCreateItemFromParsingName(
		folder,
		NULL,
		IID_IShellItem,
		reinterpret_cast<void**>(&item));
	COMHelper::TestOk(hr);

	hr = library->AddFolder(&(*item));
	COMHelper::TestOk(hr);

	hr = library->SetDefaultSaveFolder(DSFT_DETECT, &(*item));
	COMHelper::TestOk(hr);

	hr = library->Commit();
	COMHelper::TestOk(hr);

	return item;
}


int main(int argc, char* argv[])
{
	try
	{
		COMInitializer ci;

		GUID guidlib = FOLDERID_PicturesLibrary;
		// ADAPTEZ ce code  votre configuration
		WCHAR* folder = L"D:\\logos"; 

		SmartCOMPtr<IShellLibrary> library;
		SmartCOMPtr<IShellItem> itemfolder;
		
		library = GetLibrary(guidlib);
		ShowDefaultFolder(library);
		cout << endl;
		ListFolders(library);
		cout << endl;
		AddFolderAndSetDefault(library, folder);
		ShowDefaultFolder(library);
		cout << endl;
		ListFolders(library);
		cout << endl;
	}
	catch(COMException ce)
	{
		std::cout << ce.raison() << std::endl;
		return EXIT_FAILURE;
	}
	return EXIT_SUCCESS;
}
